package jwars;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.Toolkit;
import java.awt.datatransfer.StringSelection;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JTextField;

import jwars.gui.PlayerList;

/**
 * A menu which will be shown to JWars players upon running the application.
 * Allows players to create and join games across network, and finally
 * launch games if the local player is the server.
 */
public class WarMenu extends JFrame{

  private JButton createButton = new JButton("Create Server");
  private JButton copyIPButton = new JButton("Copy IP to clipboard");
  private JButton joinButton = new JButton("Join Game");
  private JButton startButton = new JButton("Launch");
  private JTextField ipField = new JTextField("localhost");
  private MenuListener listener = new MenuListener();
  private PlayerList playerList = new PlayerList();

  private ServerSession server;
  private WarSession client;

  private JComponent mainContainer = new JPanel(new BorderLayout());

  public WarMenu(){
    super("JWars Launcher");
    if(Util.useSpecialATIHack){
      this.setUndecorated(true);
    }
    this.setLayout(new BorderLayout());
    this.add(mainContainer);
    JComponent buttonContainer = new JPanel(new GridLayout(1,3));

    JComponent serverButtons = new JPanel(new GridLayout(2,1));
    serverButtons.setBorder(BorderFactory.createTitledBorder("Server"));
    serverButtons.add(createButton);
    serverButtons.add(copyIPButton);

    copyIPButton.addActionListener(new ActionListener(){
      public void actionPerformed(ActionEvent actionEvent){
        Toolkit.getDefaultToolkit().getSystemClipboard().setContents(new StringSelection(Util.getLocalIPAddress()), null);
      }
    });

    buttonContainer.add(serverButtons);

    JComponent clientButtons = new JPanel(new GridLayout(2,1));
    clientButtons.add(ipField);
    clientButtons.add(joinButton);
    clientButtons.setBorder(BorderFactory.createTitledBorder("Enter IP of server"));
    buttonContainer.add(clientButtons);

    JComponent gameButtons = new JPanel(new GridLayout(2,1));
    JButton setupButton = new JButton("Exit");
    setupButton.addActionListener(new ActionListener(){
      public void actionPerformed(ActionEvent actionEvent){
        System.exit(0);
        //Util.PROPS.store();
      }
    });
    gameButtons.setBorder(BorderFactory.createTitledBorder("Game"));
    gameButtons.add(setupButton);
    gameButtons.add(startButton);
    buttonContainer.add(gameButtons);

    this.add(buttonContainer, BorderLayout.NORTH);
    /*JComponent buttonContainer = new JPanel(new GridLayout(5,1,0,0));
    //this.setLayout(new GridLayout(5,1,0,0));
    buttonContainer.add(createButton);
    buttonContainer.add(joinButton);
    buttonContainer.add(new JLabel("Enter IP:"));
    buttonContainer.add(ipField);
    buttonContainer.add(startButton);

    this.setLayout(new GridLayout(1,2));
    this.add(buttonContainer);
    this.add(playerList);*/

    createButton.addActionListener(listener);
    joinButton.addActionListener(listener);
    ipField.addActionListener(listener);
    startButton.addActionListener(listener);

    startButton.setEnabled(false);
    ipField.setPreferredSize(new Dimension(140,0));

    this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    this.pack();
    createButton.requestFocusInWindow();
    this.setVisible(true);
  }

  public void expandInterface(JComponent component){
    playerList.setPreferredSize(new Dimension(120, 0));
    playerList.setOpaque(true);
    JComponent playerListContainer = new JPanel();
    playerListContainer.setLayout(new GridLayout(1,1));
    playerListContainer.setBorder(BorderFactory.createTitledBorder("Players"));
    playerListContainer.add(playerList);
    mainContainer.add(playerListContainer, BorderLayout.WEST);
    component.setBorder(BorderFactory.createTitledBorder("Correspondence"));
    mainContainer.add(component, BorderLayout.CENTER);
    component.setPreferredSize(new Dimension(0,240));
    mainContainer.revalidate();
    //A problem prevents proper packing when the computer is occupied
    //Reproduced by dragging window around quickly while joining or creating
    this.pack();
  }

  /**
   * Creates a server, which listens for clients.
   */
  public void create(){
    server = new ServerSession(Util.TIMER);
  }

  /**
   * Attempts to join a game created at the argument IP address.
   * @param ip String
   */
  public void join(String ip){
    client = new WarSession(this, ip, Util.getPort());
  }

  /**
   * Listener for handling menu input.
   */
  private class MenuListener implements ActionListener {

      /**
       * Manages the menu options when the user creates a server.
       * Will also join that game on the user's own computer.
       */
      public void createGame(){
      createButton.setEnabled(false);
      createButton.setText("Server Created");
      create();
      joinGame("localhost");
      startButton.setEnabled(true);
    }

    /**
     * Manages the menu option when the user creates a server and
     * invokes the method responsible for joining a game at the argument
     * IP address.
     * @param ip String
     */
    public void joinGame(String ip){
      createButton.setEnabled(false);
      joinButton.setEnabled(false);
      joinButton.setText("Client spawned");
      ipField.setEnabled(false);
      startButton.requestFocusInWindow();
      join(ip.trim());
    }

    /**
     * Sends the instruction across the
     * network that the game is commencing immediately, and instructs the
     * server to accept no more joining clients.
     *
     * This method will do nothing if the user has not created a server.
     */
    public void startGame(){
      if(server != null && client != null){
        startButton.setEnabled(false);
        startButton.setText("Game Running!");
        server.stopListening();
        client.sendLaunchInstruction();
      }
    }

    /**
     * Delegates different button
     * @param e ActionEvent
     */
    public void actionPerformed(ActionEvent e){
      Object source = e.getSource();
      if(source == createButton){
        createGame();
      } else if(source == joinButton || source == ipField){
        joinGame(ipField.getText());
      } else if(source == startButton){
        startGame();
      }
    }

  }

}
