package jwars.control.networkevents;

import jwars.net.IOHandler;
import jwars.net.Protocol;
import jwars.model.Unit;
import jwars.WarSession;
import jwars.control.Control;
import jwars.control.InputManager;

/**
 * This singleton class contains all the network instructions that are used.
 * All instructions that are included should be included here such that
 * instruction ordering is consistent no matter in which order classes are
 * loaded (if events are registered from static context and
 * in different classes, no consistent order can otherwise be guaranteed).
 */
public class NetworkEvents{

  private final static NetworkEvents instance = new NetworkEvents();

  private final Protocol protocol =
      new Protocol(WarSession.getDefaultSynchInstruction());

  private NetworkEvents(){
    protocol.addType(WarSession.getLaunchInstruction());
    protocol.addType(InputManager.getStringIOHandler());
    protocol.addType(Unit.moveHandler);
    protocol.addType(Unit.attackHandler);
    protocol.addType(Unit.stopHandler);
    protocol.lock();
  }

  public static Protocol getJWarsProtocol(){
    return instance.protocol;
  }


}
