package jwars.event;

public abstract class Event{

  final static int notScheduled = -1;
  int time = notScheduled;
  Event next; //Used for queueing
  private final EventQueue queue;

  /**
   * Constructs and Event using the argument EventQueue.
   * @param queue EventQueue
   */
  public Event(EventQueue queue){
    this.queue = queue;
  }

  /**
   * Schedule this event for execution from its EventQueue.
   */
  public void enqueue(){
    queue.add(this);
  }

  /**
   * Execute this event. Be careful using this method. It is intended to be
   * invoked from an EventQueue. Note that while an event may not be in
   * more than one queue at a time, it may legally be added to a queue
   * from this method for re-execution.
   */
  protected abstract void execute();

}
