package jwars.event.dynamic;

import jwars.synch.Timer;
import jwars.synch.TimerListener;

public class Schedule implements TimerListener {

  private DEventQueue[] queues = new DEventQueue[32];
  private int currentTime;

  public Schedule(){
    for(int i = 0; i < queues.length; i++){
      queues[i] = new DEventQueue(this, 1<<i);
    }
  }

  public void scheduleFor(DEvent e, int time){
    int delay = time - currentTime;
    schedule(e, delay);
  }

  public void schedule(DEvent e, int delay){
    if(delay<=0){
      throw new RuntimeException("Delay must be at least 1, but is "+delay);
    }
    //do other checks perhaps (though the LinkList will do some of it for us)
    e.executionTime = currentTime + delay;
    reschedule(e);
  }

  public void reschedule(DEvent e){
    int delay = e.executionTime - currentTime;
    if(delay == 0){
      e.execute();
    } else {
      int trailingZeros = Integer.numberOfLeadingZeros(delay);
      queues[31-trailingZeros].addLast(e);
    }
  }

  public void update(int frameCount, boolean isLate){
    this.currentTime = frameCount;
    for(int i = 0; i < queues.length; i++){
      queues[i].update(frameCount);
    }
  }

  public void reportStasis(){
  }

  /**
   * Tests by putting events into different queues.
   */
  public void test(){
    int testRange = 1000;
    for(int i = 1; i < (int)Math.sqrt(testRange); i++){
      DEvent e = new DEvent(){
        public void execute(){
          System.out.println("Square "+Schedule.this.currentTime);
        }
      };
      this.schedule(e, i*i);
    }
    for(int i = 1; i < testRange/2; i++){
      DEvent e = new DEvent(){
        public void execute(){
          System.out.println("Even "+Schedule.this.currentTime);
        }
      };
      this.schedule(e, 2*i);
    }

    for(int time = 0; time < testRange; time++){
      update(time, false);
      DEvent e = new DEvent(){
        public void execute(){
          System.out.println("Integer "+Schedule.this.currentTime);
        }
      };
      this.schedule(e, 1);
    }
  }

  public static void main(String[] args){
    Schedule s = new Schedule();
    s.test();
  }

}
