package jwars.gui;

import javax.swing.*;
import java.awt.*;
import jwars.model.units.*;
import jwars.model.WarModel;
import jwars.event.dynamic.DEvent;

/**
 * Displays the score of each team. This is not an actual score but rather
 * some statistics about force sizes and losses. Also, this is not actually a
 * tab but just a JPanel which happens to be displayed in a tab in the current
 * GUI.
 */
public class ScoreTab extends JPanel {

  private TeamPanel german;
  private TeamPanel soviet;
  private double secsPerUpdate = 2.0;
  private int updateInterval = WarModel.WORLD.secondsToFrames(secsPerUpdate);

  //Update every two seconds
  private DEvent update = new DEvent(){
    public void execute(){
      german.updateTallies();
      soviet.updateTallies();
      WarModel.WORLD.waitFor(this, updateInterval);
    }

  };

  /**
   * Creates a ScoreTab.
   */
  public ScoreTab(){
    BoxLayout layout = new BoxLayout(this, BoxLayout.Y_AXIS);
    setLayout(layout);
    //SpringLayout layout = new SpringLayout();
    //Spring spring = Spring.constant(1);
    //setLayout(new GridLayout(2,1,0,0));
    //setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
    add(german = new TeamPanel(Teams.GERMANY));
    add(soviet = new TeamPanel(Teams.SOVIET_UNION));

    update.execute();
  }

  /**
   * This panel displays force strength and losses for a team.
   */
  private class TeamPanel extends JPanel{

    private Team team;

    private JPanel strengthPanel = new JPanel();
    private JPanel lossPanel = new JPanel();

    private JLabel
        //strengthLabel = new JLabel("Strength"),
        //lossesLabel = new JLabel("Casualties"),
        infCasualtiesLabel = new JLabel("Infantry"),
        vehCasualtiesLabel = new JLabel("Vehicles"),
        infStrengthLabel = new JLabel("Infantry"),
        vehStrengthLabel = new JLabel("Vehicles");

    private JLabel
        vehStrTally = new JLabel("0"),
        infStrTally = new JLabel("0"),
        vehCasTally = new JLabel("0"),
        infCasTally = new JLabel("0");

    /**
     * Construct a team force/casualty display for the specified team.
     * @param team the Team
     */
    public TeamPanel(Team team) {
      this.team = team;
      strengthPanel.setBorder(BorderFactory.createTitledBorder("Force strength"));
      lossPanel.setBorder(BorderFactory.createTitledBorder("Casualties"));
      setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
      setBorder(BorderFactory.createTitledBorder(BorderFactory.createLineBorder(team.TEAM_COLOR), team.name()));

      strengthPanel.setLayout(new BoxLayout(strengthPanel, BoxLayout.Y_AXIS));
      lossPanel.setLayout(new BoxLayout(lossPanel, BoxLayout.Y_AXIS));

      lossPanel.add(createIndicator(infCasualtiesLabel, infCasTally));
      lossPanel.add(createIndicator(vehCasualtiesLabel, vehCasTally));
      strengthPanel.add(createIndicator(infStrengthLabel, infStrTally));
      strengthPanel.add(createIndicator(vehStrengthLabel, vehStrTally));

      add(strengthPanel);
      add(lossPanel);

    }

    /**
     * Creates a pair of labels meant to display the name of a value and the
     * value. (for instance [Force Strength] and [42])
     *
     * @param name JLabel
     * @param value JLabel
     * @return JPanel
     */
    private JPanel createIndicator(JLabel name, JLabel value){
      JPanel p = new JPanel();
      p.setLayout(new GridLayout(1,2,20,5));
      p.add(name);
      p.add(value);
      return p;
    }

    /**
     * Updates all the indicators for all the indicators of this TeamPanel
     */
    public void updateTallies(){
      vehStrTally.setText(Integer.toString(team.getVehicleStrength()));
      infStrTally.setText(Integer.toString(team.getInfantryStrength()));
      vehCasTally.setText(Integer.toString(team.getVehicleCasualties()));
      infCasTally.setText(Integer.toString(team.getInfantryCasualties()));
    }

  }

}
