package jwars.model.ai;

import jwars.model.Unit;

/**
 * This is the base interface which must be implemented by the AI object of any
 * Unit.
 */
public interface UnitAI
    extends MobileAI, ListeningAI, AttackAI {

  public Unit owner();

  /**
   * Empty implementation shared by all Units that have no real AI assigned
   */
  public final static UnitAI PASSIVE_AI = new UnitAI(){
    public void reportFinished(Object source){}
    public void reportFailed(Object source){}
    public void orderMove(ListeningAI source, int x, int y){}
    public void appendMove(ListeningAI source, int x, int y){}
    public void prependMove(ListeningAI source, int x, int y){}
    public boolean isMoving(){
      return false;
    }

    public void orderHalt(ListeningAI source){}
    public void setDestination(int x, int y){}


    public Unit owner(){
      return null;
    }

    public void orderAttack(Attackable target){}
    public void orderAttack(int x, int y, int radius){}
    public void orderHoldFire(){}

    public boolean isFiring(){
      return false;
    }

    public Attackable getTarget(){
      return null;
    }

    private final java.util.Iterator<Position> iter = new java.util.Iterator<Position>(){

      public boolean hasNext(){
        return false;
      }

      public Position next(){
        throw new java.util.NoSuchElementException();
      }

      public void remove(){
        throw new UnsupportedOperationException();
      }

    };

    public java.util.Iterator<Position> iterator(){
      return iter;
    }

  };

}
