package jwars.model.ai.impl;

import jwars.model.DefaultFormationPositions;
import jwars.model.Formation;
import jwars.model.Unit;
import jwars.model.ai.Attackable;
import jwars.model.ai.ListeningAI;
import jwars.model.ai.UnitAI;
import java.util.Iterator;
import jwars.model.ai.Position;

public class FormationAI implements UnitAI {

  private MovementQueueAI queueAI;

  private final Formation owner;
  private int subUnitOrdersCompleted = 0;
  private boolean isMoving = false;

  public FormationAI(Formation owner){
    this.owner = owner;
    queueAI = new MovementQueueAI(owner){
      public void setDestination(int x, int y){
        FormationAI.this.setDestination(x,y);
      }

    };
  }

  public void reportFinished(Object source){
    subUnitOrderCompleted();
  }

  public void reportFailed(Object source){
    subUnitOrderCompleted();
  }

  private void subUnitOrderCompleted(){
    subUnitOrdersCompleted++;
    if(subUnitOrdersCompleted == owner.getSubUnitCount()){
      queueAI.pumpOrders();
    }
  }

  public void orderMove(ListeningAI source, int x, int y){
    queueAI.orderMove(source, x, y);
  }

  public void appendMove(ListeningAI source, int x, int y){
    queueAI.appendMove(source, x, y);
  }

  public void prependMove(ListeningAI source, int x, int y){
    queueAI.prependMove(source, x, y);
  }

  public void orderHalt(ListeningAI source){
    queueAI.orderHalt(source);
    for(int i = 0; i < owner.getSubUnitCount(); i++){
      owner.getUnit(i).getAI().orderHalt(source);
    }
  }

  public void setDestination(int x, int y){
    owner.setDestination(x,y);
    subUnitOrdersCompleted = 0;

    double theta = queueAI.getMoveAngle();
    for(int i = 0; i < owner.getSubUnitCount(); i++){
      Unit u = owner.getUnit(i);
      int radius = u.getRadius();
      int unitX = x+(int)(DefaultFormationPositions.getX(i, theta, radius));
      int unitY = y+(int)(DefaultFormationPositions.getY(i, theta, radius));
      u.getAI().orderMove(this, unitX,unitY);
    }
  }

  public double getMoveAngle(){
    return queueAI.getMoveAngle();
  }

  public void orderAttack(Attackable target){
    for(int i = 0; i < owner.getSubUnitCount(); i++){
      owner.getUnit(i).getAI().orderAttack(target);
    }
  }

  public void orderAttack(int x, int y, int radius){
    for(int i = 0; i < owner.getSubUnitCount(); i++){
      owner.getUnit(i).getAI().orderAttack(x,y,radius);
    }
  }

  public void orderHoldFire(){
    for(int i = 0; i < owner.getSubUnitCount(); i++){
      owner.getUnit(i).getAI().orderHoldFire();
    }
  }

  /*public void setLocation(int x, int y){
    queueAI.setLocation(x,y);
  }*/

  public Unit owner(){
    return owner;
  }

  public boolean isMoving(){
    return queueAI.isMoving();
  }

  public boolean isFiring(){
    return false;
  }

  public Attackable getTarget(){
    return null;
  }

  public Iterator<Position> iterator(){
    return queueAI.iterator();
  }

}
