package jwars.model.collision;

import jwars.gui.units.Sprite;
import jwars.model.*;

/**
 * This interface should be implemented in order for objects to be registered
 * with the CollisionDetector.
 */
public interface Collider{

  public CollisionProperties getCollisionProps();

  /**
   * @return the x value of this Collider's location in main coordinates.
   */
  public int getX();

  /**
   * @return the y value of this Collider's location in main coordinates.
   */
  public int getY();

  /**
   * @return the radius of this Collider measured in the main coordinate system.
   */
  public int getRadius();

  /**
   * @return the Sprite of this Collider.
   */
  public Sprite getSprite();

  /**
   * This method is invoked when this Collider collides with the other as a
   * result of this Collider being moved.
   *
   * @param other Collider
   */
  public void collisionDetected(Collider other);

}
