package jwars.model.map.generator;

/**
 * <p>q w</p>
 *
 * <p>q w</p>
 *
 * <p>e r</p>
 *
 * <p>r t</p> y u i o
 */

import java.util.Random;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Transparency;
import java.awt.image.BufferedImage;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;

public class HeightGenerator{

  private FloatBuffer nodes;
  //private float[] nodes;
  private final int size, logSize;
  private Random rand;
  private int bitmask;

  public HeightGenerator(long seed, int logSize){
    rand = new Random(seed);
    this.logSize = logSize;
    size = (1 << logSize);
    bitmask = size - 1;
    nodes = new FloatBuffer(size, size);//new float[size * size];
  }

  public FloatBuffer generateHeights(){
    for(int step = size / 2; step > 0; step >>= 1){
      calculateDisplacements(step);
    }
    return nodes;
  }

  private void calculateDisplacements(int step){
    //Displace center points
    for(int i = step; i < size; i += 2*step){
      for(int j = step; j < size; j += 2*step){
        displaceSquare(i, j, step);
      }
    }

    //Displace corners
    for(int i = 0; i < size; i += 2*step){
      for(int j = 0; j < size; j += 2*step){
        displaceSquare(i, j, step);
      }
    }

    //Displace diamonds
    for(int j = 0; j < size; j+=2*step){
      for(int i = step; i < size; i+= 2*step){
        displaceDiamond(i,j, step);
      }
      for(int i = 0; i < size; i += 2*step){
        displaceDiamond(i, j+step, step);
      }
    }

  }

  private float next(){
    return 2 * rand.nextFloat() - 1;
  }

  private float get(int i, int j){
    return nodes.get(i&bitmask, j&bitmask);
    //return nodes[(i & bitmask) * size + (j & bitmask)];
  }

  private void set(int i, int j, float value){
    nodes.set(i&bitmask, j&bitmask, value);
    //nodes[(i & bitmask) * size + (j & bitmask)] = value;
  }

  private void displaceSquare(int i, int j, int step){
    float neighbourSum =
        get(i - step, j - step) + get(i - step, j + step) +
        get(i + step, j - step) + get(i + step, j + step);
    set(i, j, neighbourSum / 4 + next());
  }

  private void displaceDiamond(int i, int j, int step){
    float neighbourSum =
        get(i - step, j) + get(i + step, j) +
        get(i, j - step) + get(i, j + step);
    set(i, j, neighbourSum / 4 + next());
  }

  /*public static float[] crop(float[] source, int width, int height){
    for(int i = 0; i<
  }*/

  public static void main(String[] args){
    int bits = 5;
    long seed = 0;
    if(args.length > 0){
      bits = Integer.valueOf(args[0]);
      seed = Long.valueOf(args[1]);
    }
    HeightGenerator h = new HeightGenerator(seed, bits);
    FloatBuffer map = h.generateHeights();
    map.scaleToFit(-1,1);
    map.smoothify();
    map.smoothify();
    map.smoothify();
    float maxHeight = -Float.POSITIVE_INFINITY;
    for(int i = 0; i < map.SIZE; i++){
      if(Math.abs(map.get(i)) > maxHeight){
        maxHeight = Math.abs(map.get(i));
      }
    }
    System.out.println("Maximum height "+maxHeight);
    BufferedImage im = new BufferedImage(map.WIDTH, map.HEIGHT, Transparency.OPAQUE);
    Graphics g = im.getGraphics();
    for(int i=0; i < map.WIDTH; i++){
      for(int j = 0; j < map.HEIGHT; j++){
        float height = map.get(i, j);
        Color c;
        try{
          if(height > 0){
            c = new Color(height / maxHeight, 0, 0);
          } else{
            c = new Color(0, 0, -height / maxHeight);
          }
          g.setColor(c);
          g.drawLine(i, j, i, j);
        }
        catch(Exception ex){
          ex.printStackTrace();
          System.out.println("height "+height+ "maxh "+maxHeight);
          System.out.println("i j "+i+" "+j);
          System.exit(0);
        }

      }
    }
    JFrame frame = new JFrame();
    frame.add(new JLabel(new ImageIcon(im)));
    frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    frame.pack();
    frame.setVisible(true);
    System.out.println("Size "+map.WIDTH+ " x "+map.HEIGHT);

  }

}
