package jwars.model.pathfinder;

import jwars.model.Coord;
import java.util.List;
import java.util.ArrayList;
import jwars.model.structures.PathFindingNode;
import jwars.model.map.*;
import jwars.model.collision.CollisionTile;
import jwars.model.World;

/**
 * <p>q w</p>
 *
 * <p>q w</p>
 *
 * <p>e r</p>
 *
 * <p>r t</p> y u i o
 */
public class LineDrawCapableMap
    extends TileMap<CollisionTile>
{
  private List<CollisionTile> tempList = new ArrayList<CollisionTile>(0);
  int counter;

  public LineDrawCapableMap(TileMap<CollisionTile> map){
    super(map);
  }

  /**
   * returns an array of coordinates equal to drawing the line though the tile
   * array using bresenham's line drawing algorithm
   * @param start Coord
   * @param end Coord
   * @return T[]
   */
  public List<CollisionTile> getTiles(Coord start, Coord end, int thickness){
    clear();
   // System.out.println("x0 ="+start.x+", y0 = "+start.y+", x1 = "+end.x+", y1 = "+end.y);
   // System.out.println("SCALE = "+scale);
    int x0 = start.x/scale;
    int y0 = start.y/scale;
    int x1 = end.x/scale;
    int y1 = end.y/scale;
    int buffer;
   // System.out.println("x0 ="+x0+", y0 = "+y0+", x1 = "+x1+", y1 = "+y1);
    counter = 0;
    boolean steep = Math.abs(y1 - y0) > Math.abs(x1 - x0);
    if (steep){
   //    System.out.println("Steep line!!!");
       //swap(x0, y0);
       buffer = x0;
       x0 = y0;
       y0 = buffer;

       //swap(x1, y1);
       buffer = x1;
       x1 = y1;
       y1 = buffer;
     }
     if (x0 > x1){
       // System.out.println("swapping coords!!!");
       // swap(x0, x1);
       buffer = x0;
       x0 = x1;
       x1 = buffer;

       //swap(y0, y1);
       buffer = y0;
       y0 = y1;
       y1 = buffer;
    }
  //  System.out.println("x0 ="+x0+", y0 = "+y0+", x1 = "+x1+", y1 = "+y1);
    int deltax = x1 - x0;
    int deltay = Math.abs(y1 - y0);
    int error = 0;
    // ????
    int ystep = 0;
    int y = y0;
    if (y0 < y1) ystep = 1;
    else ystep = -1;

    for(int x = x0; x <= x1; x++){
      if(steep == true){
        addTile(y, x, steep, thickness);
      }
      else{
        addTile(x, y, steep, thickness);
      }
      error = error + deltay;
      if(2 * error >= deltax){
        y = y + ystep;
        error = error - deltax;
      }
    }
 //   System.out.println("tempList length = "+tempList.size());
    return tempList;
  }

  /**
   * ensures the thickness required for the check.
   * @param x int
   * @param y int
   * @param steep boolean
   * @param thick int
   */
  private void addTile(int x, int y, boolean steep, int thick){
    if(steep == true){
      for(int i = -thick; i <= thick; i++){
    //    System.out.println("x = " + (x+i) + ", y = " +y);
        tempList.add(getTile((x+i), y));
        counter++;

      }
    }
    else{
      for(int i = -thick; i <= thick; i++){
   //     System.out.println("x = " + x + ", y = " + (y + i));
        tempList.add(getTile(x,(y+i)));
        counter++;
      }
    }
    //System.out.println(counter);
  }

  public void clear(){
    tempList.clear();
  }
}
