package jwars.model.pathfinder;

import jwars.model.structures.TerrainObject;
import java.util.Arrays;
import jwars.model.map.*;
import java.math.*;
import jwars.model.Coord;
import java.util.*;
import jwars.model.collision.Collider;
import jwars.model.structures.*;
import jwars.model.collision.CollisionTile;
import jwars.model.pathfinder.LineDrawCapableMap;
import jwars.model.Unit;
import jwars.model.structures.PathFindingNode;
import jwars.model.World;
import jwars.model.collision.CollisionProperties;
import jwars.gui.units.Sprite;

public class PathFinder{

  public final static boolean logging = false;
  private PathFindingNode start;
  private Target end;
  private int listSize = 100;
  private int thickness;
  private boolean succes;
  private LineDrawCapableMap ldcMap;
  private NodeList openList;
  private NodeList closedList;
  private List<Coord> orders;
  private List<Collider> objects;

  public PathFinder(TileMap<CollisionTile> colMap, World world){

    /*bluep = new BluePrint(true, 1);
         end = new TerrainObject(bluep, 0, 0, 0);*/
    end = new Target();
    start = new Target();
    //  world.DETECTOR.register(end);

    ldcMap = new LineDrawCapableMap(colMap);
    openList = new NodeList(listSize);
    closedList = new NodeList(listSize);
    orders = new ArrayList<Coord>(0);
    objects = new ArrayList<Collider>(0);
  }

  public List<Coord> findPath(Coord endc, Unit unit){
    if(logging)
      System.out.println("Finding path");

    PathFindingNode current;
    thickness = 1; //udregn thickness fra maincord/ldcMapsize
    //(den strste)+1 og del med 2;

    if(logging)
      System.out.println("Moving the target zone");
    end.set(endc.x, endc.y);
    ldcMap.getTileAt(end.x, end.y).register(end);

    start.set(unit.getX(), unit.getY());
    start.setHnull();
    openList.addNode(start);
    closedList.addNode(start);
    current = openList.popHead();

    succes = false;
    if(logging)
      System.out.println("starting algorithm");
    int i = 0;
    do{
      expand(current, end, unit);
      current = openList.popHead();
      if(current == end){
        ldcMap.getTileAt(end.x, end.y).remove(end);
        if(logging)
          System.out.println("We hit the end target - BACKTRACKING");
        backtrack();
        clean();
        succes = true;
      }
      i++;
      if(logging)
        System.out.println("* * * * * Nr " + i + " popping of head * * * * * ");
    } while(!openList.isEmpty());
    // Vejen er nu fundet og der afsluttes eller der kunne ikke expandes og der
    // er fejl over det hele
    if(succes){
      if(logging)
        System.out.println("Vejen er fundet!!!");
    } else{
      System.out.println("FEJL I PATHFINDEREN!");
    }
    ldcMap.getTileAt(endc.x, endc.y).remove(end);
    ArrayList<Coord> returnValue = new ArrayList<Coord>(orders.size());
    returnValue.addAll(orders);
    return returnValue;
  }

  private void expand(PathFindingNode from, PathFindingNode to, Unit unit){
    if(logging)
      System.out.println("Expanding");
    PathFindingNode expand1 = null; // = from.calculateDestinationFromCorner(unit.getRadius());
    PathFindingNode expand2 = null; // = to.calculateDestinationFromCorner(unit.getRadius());
    Coord startCoord = from.calculateDestinationFromCorner(unit.getRadius());
    Coord endCoord = to.calculateDestinationFromCorner(unit.getRadius());
    if(logging)
      System.out.println(" Starting Coord " + startCoord.toString());
    if(logging)
      System.out.println(" Finish Coord " + endCoord.toString());

    // thickness mangler at blive udregnet - default = 1
    List<CollisionTile> tiles = ldcMap.getTiles(startCoord, endCoord, thickness);
    List<Collider> terrainObjects = getObjects(tiles);

    double maxAngle = -5;
    double minAngle = 5;
    double angle = 0;

    // For each terrain object
    for(int j = 0; j < terrainObjects.size(); j++){
      // System.out.println("fundet en collider");
      if(terrainObjects.get(j) instanceof Target){
        if(logging)
          System.out.println(
              "******* ADDED ^^ TARGET ^^ TO OPEN/CLOSED LISTS *******");
        end.update(from, end);
        openList.addNode(end);
        return;
      }

      TerrainObject TObject = (TerrainObject)(terrainObjects.get(j));

      //  System.out.println("Kontrollerer en bygning");
      // lb strucpoints igennem HVIS linjen er indenfor radius
      maxAngle = 0;
      minAngle = 0;
      // Is the line within reach of the objects radius
      int dist = (int)(pointLineDist(startCoord, endCoord,
          terrainObjects.get(j).getX(),
          terrainObjects.get(j).getY()));
      int rad = 2 * unit.getRadius() + terrainObjects.get(j).getRadius();

      if(dist - 5 <= rad){
        if(logging)
          System.out.println("INDENFOR bygningens radius");
        // For hvert punkt i strukturen
        for(int k = 0; k < TObject.structPoints.length; k++){

          if(TObject.structPoints[k] == to){
            if(to.update(from, end)){
              if(logging)
                System.out.println("******* ADDED TO OPEN/CLOSED LISTS *******");
              openList.addNode(to);
              closedList.addNode(to);
              return;
            }
          } else{
            //    System.out.println("Ikke mlet \nKontrollerer er structpoint : "+k);
            angle = getAngle(startCoord, endCoord,
                             TObject.getStructPoint(k), true);
            //    System.out.println("Den forskudte - 90* - vinkel er mlt til (cos) :" + angle);
            if(Math.abs(angle) >= Math.PI / 2){
              //     System.out.println("punktet ligger til VENSTRE for linjen");
              angle = getAngle(startCoord, endCoord,
                  TObject.getStructPoint(k), false);
              //     System.out.println("Vinkel til strucpoint = " + angle);
              if(angle > minAngle){
                expand1 = TObject.getStructPoint(k);
                if(logging)
                  System.out.println("expand1 er sat til structpoint nr :" + k);
                minAngle = angle;
              }
            } else{
              // System.out.println("punktet ligger til HJRE for linjen");
              angle = getAngle(startCoord, endCoord,
                  TObject.getStructPoint(k), false);
              //   System.out.println("Vinkel til strucpoint = " + angle);
              if(angle > maxAngle){
                expand2 = TObject.getStructPoint(k);
                if(logging)
                  System.out.println("expand2 er sat til structpoint nr :" + k);
                maxAngle = angle;
              }
            }
          }
        } // For each structpoint
        // Is the line between the two angles and both angles is in the forward direction
        if(minAngle > 0 && maxAngle > 0 &&
           (Math.abs(minAngle) < Math.PI / 2 &&
            Math.abs(minAngle) < Math.PI / 2)){
          if(logging)
            System.out.println("Linjen gr imellem max/min - vi udvider");
          expand(from, expand1, unit);
          expand(from, expand2, unit);
        } else{
          if(logging)
            System.out.println(
                "Linjen gr IKKE imellem max/min");

        }

      } // Hvis den er indenfor radius
      if(logging)
        System.out.println("Udenfor bygningens radius");
    } // hver collider
  }

  /**
   * Used for first backtracking through the pathfindingnodes setting the baby
   * at each pass and then tracking through appending the moves on the
   * selected unit
   * @param unit Unit
   */
  private void backtrack(){
    // Clearing the order list for this run
    orders.clear();
    PathFindingNode current = end;
    int counter = 0;
    do{
      current.ancestor.setBaby(current);
      current = current.ancestor;
      counter++;
    } while(current != start);
    do{
      orders.add(current.getIndented());
      current = current.getBaby(); ;
    } while(current != end);

    orders.add(end.getIndented());
    if(logging)
      System.out.println("Total of " + counter +
                       " pathfindingnodes in the list of orders");
  }

  /**
   * Returns a list of TerrainObjects that exists in the given list of collision
   * tiles. No TerrainObject will be added more than once.
   * @param list List
   * @return List
   */
  private List<Collider> getObjects(List<CollisionTile> list){
    objects.clear();
    int counter = 0;
    boolean endline = false;
    for(int i = 0; i < list.size(); i++){
      // System.out.println("Checking tile for objects to contorl");
      for(int j = 0; j < list.get(i).colliders.size(); j++){
        if(list.get(i).colliders.get(j) instanceof TerrainObject){
          TerrainObject TObject = (TerrainObject)list.get(i).colliders.get(j);
          if(objects.contains(TObject)){
            // Already in the object list
            //    System.out.println("Object already found once");
          } else{
            if(logging)
              System.out.println("Adding a building!");
            objects.add(TObject);
            counter++;
          }
        }
        if(list.get(i).colliders.get(j) instanceof Target){
          if(logging)
            System.out.println("Adding target");
          endline = true;
        }
      }
    }
    if(endline){
      objects.add(end);
      counter++;
    }
//    System.out.println("Total amount of objects on the line : " +
    //                      objects.size()+"\nobjects added this run = "+counter);
    return objects;
  }

  private void clean(){
    closedList.reset();
    openList.clear();
    closedList.clear();
    end.reset();
  }

  /**
   * This class is used by the pathfinder for the end/start locations. Used
   * only for calculation purposes.
   * <p>q w</p>
   *
   * <p>q w</p>
   *
   * <p>e r</p>
   *
   * <p>r t</p> y u i o
   */
  private class Target
      extends PathFindingNode implements Collider{

    private CollisionProperties colprop;

    public Target(){
      super();
      colprop = new CollisionProperties(this, false, false);
    }

    public CollisionProperties getCollisionProps(){
      return colprop;
    }

    public Coord calculateDestinationFromCorner(int radius){
      return this;
    }

    public int getX(){
      return x;
    }

    public int getY(){
      return y;
    }

    public int getRadius(){
      return 0;
    }

    public Coord getIndented(){
      return this;
    }

    public Sprite getSprite(){
      return Sprite.INVISIBLE;
    }

    public void collisionDetected(Collider other){
    }
  }

  /**
   * The internal class NodeList is needed for containing PathFdindingNodes
   * for the pathfinder. In the algorithm used we need both the closedList
   * and the openList.
   */
  private static class NodeList{
    final PathFindingNode[] node;
    int count;
    boolean f_sorted;

    public NodeList(int size){
      node = new PathFindingNode[size];
    }

    // Add a node. Returns false if there's no room.
    boolean addNode(PathFindingNode newNode){
      // System.out.println("Tilfjer node til openlist/closedList");
      if(count == node.length)
        return false;
      f_sorted = false;
      node[count++] = newNode;

      return true;
    }

    // Remove a node from the list
    void removeNode(Coord co){
      int index = findNode(co);
      assert index != -1;
      f_sorted = false;
      node[index] = node[--count];
      node[count] = null;
    }

    void sort(){
      Arrays.sort(node);
      f_sorted = true;
    }

    // Find a node's position by its coordinates. Returns -1 if node is not found.
    int findNode(Coord newcoord){

      for(int i = 0; i < count; i++)
        if(node[i].equals(newcoord))
          return i;

      return -1;
    }

    // Return a node by its coordinates (or null if not present)
    PathFindingNode getNode(Coord coord){
      int index = findNode(coord);
      if(index == -1)
        return null;
      else
        return node[index];
    }

    // clear the list - used for resetting the pathfinder
    void clear(){
      for(int i = 0; i < count; i++)
        node[i] = null;
      count = 0;
    }

    int size(){
      return count;
    }

    void reset(){
      for(int i = 0; i < node.length; i++){
        if(node[i] != null){
          node[i].reset();
        }
      }
    }

    boolean isEmpty(){
      return count == 0;
    }

    // Pop the head of the list
    PathFindingNode popHead(){
      if(count > 0 && !f_sorted){
        Arrays.sort(node, 0, count);
        f_sorted = true;
      }
      PathFindingNode ret = node[0];
      node[0] = node[--count];
      node[count] = null;
      return ret;
    }

  }

  /**
   * We use the scalar product for calculating the angle.
   *
   * Angle = arccos ( a * b / ( |a|*|b| ) )
   *
   * Returns the angle from the vector src:reference to src:target
   *
   * @param src Coord
   * @param target Coord
   * @return double
   */

  public static double getAngle(Coord src, Coord reference, Coord target,
      boolean displace){
    double refX = reference.x - src.x;
    double refY = reference.y - src.y;
    double tarX = target.x - src.x;
    double tarY = target.y - src.y;
    if(displace){
      double buffer = tarX;
      tarX = tarY;
      tarY = -buffer;
    }
    double enumerator = refX * tarX + refY * tarY;
    double denominator = dist(refX, refY) * dist(tarX, tarY);

    return Math.acos(enumerator / denominator);
  }

  public static double dist(double x, double y){
    return Math.hypot(x, y);
  }

  /**
   *                                  2                         2
   qx r1x - qx r0x - r0x r1x + r0x  - qy r0y - r0y r1y + r0y  + qy r1y
   -------------------------------------------------------------------
             2                  2      2                  2
          r1x  - 2 r0x r1x + r0x  + r1y  - 2 r0y r1y + r0y

     public static double pointLineDist(Coord src, Coord dst, Coord point){
    double
        qx = point.x, qy = point.y,
        r0x = src.x, r0y = src.y,
        r1x = dst.x, r1y = dst.y;
    double enumerator = qx * (r1x - r0x) - r0x * r1x + r0x * r0x -
        qy * (r0y - r1y) - r0y * r1y + r0y * r0y;
   double denominator = (r1x - r0x) * (r1x - r0x) + (r1y - r0y) * (r1y - r0y);
    double t = enumerator / denominator;
    double x = r0x + t * (r1x - r0x);
    double y = r0y + t * (r1y - r1x);
    double distSqr = (x - qx) * (x - qx) + (y - qy) * (y - qy);
    return Math.sqrt(distSqr);
     }*/

  public static double pointLineDist(Coord src, Coord dst, int x0, int y0){
    return pointLineDist(src.x, src.y, dst.x, dst.y, x0, y0);
  }

  /**
   * Calculate the distance between the line segment defined by the points
   * (x1,y1) and (x2,y2) to the point (px,y).
   *
   * @param x1 double
   * @param y1 double
   * @param x2 double
   * @param y2 double
   * @param px double
   * @param py double
   * @return the distance
   */
  public static double pointLineDist(double x1, double y1, double x2,
      double y2, double px, double py){
    return jwars.Util.pointLineDist(x1, y1, x2, y2, px, py);
  }

  private static void getStackTrace(String s){
    try{
      throw new Exception(s);
    } catch(Exception ex){
      ex.printStackTrace();
      System.exit(0);
    }
  }

}
