package jwars.model.structures;

import java.awt.Graphics2D;
import java.awt.Graphics2D;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Image;

import jwars.model.WarModel;
import jwars.gui.units.Sprite;
import jwars.model.collision.Collider;
import jwars.model.collision.CollisionProperties;
import jwars.model.Coord;

import jwars.model.structures.PathFindingNode;
import jwars.Util;

public class TerrainObject
    implements Collider{

  //private Image image;

  public PathFindingNode[] structPoints;
  public Coord location = new Coord();
  private final TerrainSprite sprite;
  private final CollisionProperties collisionProps;
  private int radius;
  //private double angle;
  public boolean small;

  public TerrainObject(Image image, BluePrint bluePrint, double angle, int x,
                       int y){

    //this.angle = angle;
    //this.image = image;
    this.radius = bluePrint.getRadius();
    //  System.out.println("building radius set to: "+radius);
    if(radius < 500){
      small = true;
    } else small = false;

    //for(PathFindingNode c : structPoints){
      //  System.out.println("1 GENNEMGANG");
    //  c.rotate(angle).plus(location);
    //}

    this.location.set(x, y);

    structPoints = createPathFindingNodes(bluePrint.convexHull, angle, x, y);

    collisionProps = new CollisionProperties(this, bluePrint.getMassive(),
        bluePrint.getMoveable()){
      public boolean checkCollide(Collider other, int x, int y){
        int r = other.getRadius();
        Coord currentPoint; // = structPoints[structPoints.length - 1];
        Coord nextPoint; // = structPoints[0];
        for(int i = 0; i < structPoints.length - 1; i++){
          currentPoint = structPoints[i];
          nextPoint = structPoints[i + 1];
          if(checkSegment(currentPoint, nextPoint, x, y, r)){
            return true;
          }
        }
        return checkSegment(structPoints[structPoints.length - 1],
                            structPoints[0], x, y, r);
      }
    };

    sprite = new TerrainSprite(getRadius());
  }

  private static boolean checkSegment(Coord c1, Coord c2, int x, int y, int r){
    return Util.pointLineDist(c1.x, c1.y, c2.x, c2.y, x, y) < r;
  }

  // Used for testing
  /*public TerrainObject(BluePrint bluePrint, double angle, int x,
                       int y){
    //this.angle = angle;
    this.radius = bluePrint.getRadius();
    if(radius < 500){
      small = true;
    } else small = false;
    this.location.set(x, y);
    this.collisionProps = new CollisionProperties(this, bluePrint.getMassive(),
        bluePrint.getMoveable());

    //this.structPoints = createPathFindingNodes(bluePrint.convexHull);
    sprite = new TerrainSprite(getRadius());
  }*/

  /**
   * The method sets up pathfindingnodes for the current building based
   * on the coordinates provided by the programmer. It also sets indentedDirections
   * for the pathfinder.
   * @param points Coord[]
   */
  private PathFindingNode[] createPathFindingNodes(Coord[] points, double angle, int x, int y){
    PathFindingNode[] structPoints = new PathFindingNode[points.length];
    Coord temporary = new Coord();
    //indentDirection = new Coord();
    for(int i = 0; i < points.length; i++){
      temporary.set(points[i]);
      structPoints[i] = new PathFindingNode(temporary.rotate(angle).plus(x,y));
    }
    //Calculate indent direction for the first point
    setIndentDirectionFor(structPoints[structPoints.length-1],
                          structPoints[0], structPoints[1]);
    for(int i=1; i < structPoints.length-1; i++){
      //....all those in the middle of the array
      setIndentDirectionFor(structPoints[i-1],
                            structPoints[i], structPoints[i+1]);
    }
    //...and the last point.
    setIndentDirectionFor(structPoints[structPoints.length-2],
                          structPoints[structPoints.length-1], structPoints[0]);
    return structPoints;
  }

  private void setIndentDirectionFor(PathFindingNode last, PathFindingNode current, PathFindingNode next){
    Coord nextSide = new Coord(next).minus(current).rotate(-Math.PI/2);
    Coord lastSide = new Coord(current).minus(last).rotate(-Math.PI/2);
    //next.plus(last);
    double nextXnorm = nextSide.x / (double)nextSide.length();
    double nextYnorm = nextSide.y / (double)nextSide.length();

    double lastXnorm = lastSide.x / (double)lastSide.length();
    double lastYnorm = lastSide.y / (double)lastSide.length();
    double indentX = lastXnorm + nextXnorm;
    double indentY = lastYnorm + nextYnorm;
    current.setIndentDirection(indentX, indentY);
  }

  public PathFindingNode getStructPoint(int i){
    return structPoints[i];
  }

  public CollisionProperties getCollisionProps(){
    return collisionProps;
  }

  public int getX(){
    return location.x;
  }

  public int getY(){
    return location.y;
  }

  public int getRadius(){
    return radius;
  }

  public Sprite getSprite(){
    return sprite;
  }

  public void moveObject(Coord newC){
    location.set(newC);
  }

  public void collisionDetected(Collider other){
  }

  public String toString(){
    return "location = (" + location.x + ", " + location.y + "), radius = " +
        this.radius;
  }

  private class TerrainSprite
      extends Sprite{

    int[] xPoints, yPoints;
    int[] ix, iy; //visualization of indent direction

    public TerrainSprite(int radius){
      super(radius);

      xPoints = new int[structPoints.length];
      yPoints = new int[structPoints.length];
      ix = new int[structPoints.length];
      iy = new int[structPoints.length];
      for(int i = 0; i < structPoints.length; i++){
        xPoints[i] = structPoints[i].x - location.x;
        yPoints[i] = structPoints[i].y - location.y;
        //The following lines have to do with visualization of indent directions
        structPoints[i].calculateDestinationFromCorner(10000);
        ix[i] = structPoints[i].getIndented().x - location.x;
        iy[i] = structPoints[i].getIndented().y - location.y;
      }
      WarModel.WORLD.mainCoords.transformToCoords(WarModel.PIXEL_COORDS,
          xPoints);
      WarModel.WORLD.mainCoords.transformToCoords(WarModel.PIXEL_COORDS,
          yPoints);
      WarModel.WORLD.mainCoords.transformToCoords(WarModel.PIXEL_COORDS,
          ix);
      WarModel.WORLD.mainCoords.transformToCoords(WarModel.PIXEL_COORDS,
          iy);
    }

    public void draw(Graphics2D g2d, int x, int y){
      int shadowWidth = 4;
      int shadowHeight = 3;
      g2d.translate(x - shadowWidth, y + shadowHeight);
      g2d.setColor(Color.black);
      g2d.fillPolygon(xPoints, yPoints, xPoints.length);
      g2d.translate(shadowWidth, -shadowHeight);
      g2d.setColor(Color.darkGray);
      g2d.fillPolygon(xPoints, yPoints, xPoints.length);
      //The following will draw a representation of structure indentations
      /*g2d.setColor(Color.red);
      for(int i = 0; i < structPoints.length; i++){
        //Coord c = n.calculateDestinationFromCorner(100000);

        g2d.drawLine(xPoints[i], yPoints[i], ix[i], iy[i]);
      }*/
    }

    /* public void draw(Graphics2D g2d, int x, int y){
       g2d.rotate(angle, x, y);
       g2d.drawImage(image, x, y, null);
     }*/

  }
}

/*public class TerrainObject
    implements Collider{

  private Image image;

  private Coord[] structPoints;
  private Coord location = new Coord();
  private TerrainSprite sprite;
  private CollisionProperties collisionProps;
  private int radius;
  private double angle;
  public boolean small;

  public TerrainObject(Image image, BluePrint bluePrint, double angle, int x,
                       int y){

    this.angle = angle;
    this.image = image;
    this.radius = bluePrint.getRadius();
    if(radius < 500){
      small = true;
    }
    else small = false;

    this.location.set(x, y);

    radius = 0;
    collisionProps = new CollisionProperties(this, bluePrint.getMassive(),
        bluePrint.getMoveable());

    for(Coord c : structPoints = bluePrint.createStructure()){
      c.rotate(angle).plus(location);
    }

    sprite = new TerrainSprite(getRadius());
  }

  public CollisionProperties getCollisionProps(){
    return collisionProps;
  }

  public int getX(){
    return location.x;
  }

  public int getY(){
    return location.y;
  }

  public int getRadius(){
    return radius;
  }

  public Sprite getSprite(){
    return sprite;
  }

  public void collisionDetected(Collider other){
  }

  private class TerrainSprite
      extends Sprite{

    public TerrainSprite(int radius){
      super(radius);

      /*   xPoints = new int[structPoints.length];
          yPoints = new int[structPoints.length];
          for(int i = 0; i < structPoints.length; i++){
            xPoints[i] = structPoints[i].x - location.x;
            yPoints[i] = structPoints[i].y - location.y;
          }
          WarModel.WORLD.MAIN_COORDS.transformToCoords(WarModel.PIXEL_COORDS,
              xPoints);
          WarModel.WORLD.MAIN_COORDS.transformToCoords(WarModel.PIXEL_COORDS,
              yPoints);*/
 /*}
      public void draw(Graphics2D g2d, int x, int y){
   g2d.rotate(angle, x, y);
   g2d.drawImage(image, x, y, null);
      }

    }
  }*/
