package jwars.model.units;

import jwars.Util;
import jwars.data.AssaultGunCategory;
import jwars.model.ai.UnitAI;
import jwars.model.ai.Attackable;
import jwars.model.ai.impl.MoveableAI;

public class AssaultGun extends Vehicle {

  private AssaultGunCategory CAT;

  public AssaultGun(AssaultGunCategory cat){
    super(cat, cat.hull, cat.superStructure);
    this.CAT = cat;
  }

  public void update(){
    MoveableAI ai = (MoveableAI)getAI();
    boolean canMove = true;
    if(ai.isFiring() && weaponry.canFire()){
      int x = ai.getTargetX();
      int y = ai.getTargetY();
      int maxRange = weaponry.maxRange();
      boolean withinRange = Util.distSqr(location.x, location.y, x, y) < (float)maxRange * maxRange;
      if(withinRange){
        super.updateFiring();
        canMove = false;
      }
    }
    if(canMove && ai.isMoving()){
      move();
    }
  }

  public boolean aimTowards(double theta){
    double angleDiff = Util.minusPiToPi(theta - this.theta);
    dirtify();
    double maxRotation = CAT.turnRate * dt;
    if(maxRotation > Math.abs(angleDiff)){
      this.theta = theta;
      return true;
    } else {
      this.theta += maxRotation * Math.signum(angleDiff);
      return false;
    }
  }


}
