package jwars.model.units;

import java.util.ArrayList;
import java.awt.Color;
import java.awt.image.BufferedImage;
import javax.swing.ImageIcon;
import jwars.model.WorldListener;
import jwars.model.Moveable;
import jwars.model.Formation;
import jwars.model.Unit;

public class Team
    extends Formation {

  private Team opponent;
  private int score=42; //not used yet, but this is what this class is (in part) about

  public final String NAME;
  public final String NAME_SHORT;
  public final String NAME_NOUN;
  public final String NAME_ADJECTIVE;
  public final Color TEAM_COLOR;
  public final ImageIcon FLAG;

  private int vehicleStrength;
  private int infantryStrength;
  private int vehicleCasualties;
  private int infantryCasualties;

  public Team(String name, String shortName, String noun, String adjective, BufferedImage flag, Color color){
    super(name, Integer.MAX_VALUE);
    this.NAME = name;
    this.FLAG = new ImageIcon(flag);
    this.NAME_SHORT = shortName;
    this.NAME_NOUN = noun;
    this.NAME_ADJECTIVE = adjective;
    this.TEAM_COLOR = color;
  }

  public void setOpponent(Team opponent){
    this.opponent = opponent;
  }

  public boolean enemyOf(Team other){
    return this != other; //Presently all teams are opponents
  }

  public Team opponent(){
    return opponent; //Presently only one opponent
  }

  public int getScore(){
    return score;
  }

  public Team getTeam(){
    return this;
  }

  public boolean hasLOS(Moveable moveable){
    return moveable.subordinateOf(this) || moveable.hasBeenSpotted(null);
  }

  public boolean isSelectable(){
    return false;
  }

  public void infantryAdded(InfantrySquad infantry){
    this.infantryStrength += infantry.getStrength();
  }

  public void vehicleAdded(Vehicle vehicle){
    this.vehicleStrength++;
  }

  public void addCasualties(int casualties){
    this.infantryCasualties += casualties;
    this.infantryStrength -= casualties;
  }

  public void vehicleDestroyed(Vehicle vehicle){
    this.vehicleCasualties++;
    this.vehicleStrength--;
  }

  public int getVehicleStrength(){
    return this.vehicleStrength;
  }

  public int getVehicleCasualties(){
    return this.vehicleCasualties;
  }

  public int getInfantryStrength(){
    return this.infantryStrength;
  }

  public int getInfantryCasualties(){
    return this.infantryCasualties;
  }

}
