package jwars.model.units;

import java.awt.Graphics2D;
import java.awt.Image;

import jwars.data.HullCategory;
import jwars.data.MoveableCategory;
import jwars.data.TurretCategory;
import jwars.gui.units.ImageSprite;
import jwars.model.Moveable;
import jwars.model.weapons.ArmourModel;
import jwars.model.weapons.VehicleArmourModel;
import jwars.test.explosions.Explosion;
import jwars.test.explosions.ImpactAnimationType;

public class Vehicle extends Moveable {

  private final VehicleArmourModel armour;

  public Vehicle(MoveableCategory cat, HullCategory hull, HullCategory superStructure){
    super(cat, true);
    this.setSprite(new VehicleSprite(hull.graphics, superStructure.graphics));
    this.armour = new VehicleArmourModel(this, hull.armour, superStructure.armour);
  }

  public Vehicle(MoveableCategory cat, HullCategory hull, TurretCategory turret){
    super(cat, true);
    this.setSprite(new VehicleSprite(hull.graphics, turret.graphics));
    this.armour = new VehicleArmourModel(this, hull.armour, turret.armour);
  }

  public void destroy(){
    world.addDynamicSprite(new Explosion(ImpactAnimationType.HUGE, location.x, location.y));
    getTeam().vehicleDestroyed(this);
    super.destroy();
  }

  /**
   * Vehicles may optionally have a "turret" which can turn. This method returns
   * the angle of the turret with respect to this vehicle, which defaults to 0.
   *
   * Override to implement traversable turret.
   *
   * @return the angle of the turret
   */
  public double getTurretAngle(){
    return 0;
  }

  public VehicleArmourModel getArmourModel(){
    return armour;
  }

  public void addToWorld(int x, int y, boolean addChildren){
    super.addToWorld(x,y,addChildren);
    this.getTeam().vehicleAdded(this);
  }

  protected class VehicleSprite extends ImageSprite{

    private java.awt.Image turretGraphics;

    public VehicleSprite(Image hullGraphics, Image turretGraphics) {
      super(hullGraphics);
      super.setRadius(Math.max(getImageRadius(hullGraphics), getImageRadius(turretGraphics)));
      this.turretGraphics = turretGraphics;
    }

    public void draw(Graphics2D g, int x, int y){
      //System.out.println("drawing tank");
      super.draw(g, x, y);
      //g.translate(x-turret.getWidth()/2, y-turret.getHeight()/2);
      g.rotate(Vehicle.this.getTurretAngle(), x, y);
      g.drawImage(turretGraphics, x-turretGraphics.getWidth(null)/2, y-turretGraphics.getHeight(null)/2, null);
    }

    public double getAngle(){
      return Vehicle.this.getAngle();
    }

  }

}
