package jwars.model.weapons;

import jwars.model.weapons.Weapon;

public interface ArmourModel{

    /**
     * When a weapon fires, the hit location is calculated, and this method will
     * be invoked in the ArmourModels of any Moveables in the vicinity of the
     * impact location.
     *
     * This method may be invoked even if hit location is far from the target,
     * which might be useful if reactions of the targetted units are desired.
     * For example, it would be useful to calculate levels of suppression based
     * on invocations on this method.
     *
     * @param weapon the weapon which fired
     * @param impactX the x coordinate of the hit location
     * @param impactY the y coordinate of the hit location
     * @param impactDist the distance between hit location and Moveable
     * @param travellingRange the firing range
     * @param impactAngle the angle of incidence on the Moveable, where 0
     * denotes a front hit.
     */
    public void reportImpact(Weapon weapon, int impactX, int impactY,
                           float impactDist, float travellingRange,
                           float impactAngle);

  /**
   * Returns whether this ArmourModel is considered vulnerable to the argument
   * weapon. This method can be used to determine which targets are better.
   *
   * NOTE: The specification of this method is incomplete. Should it take into
   * account accuracy or just armour penetration range dependence?
   *
   * @param weapon Weapon
   * @param range float
   * @param angle float
   * @return boolean
   */
  public float vulnerability(Weapon weapon, float range, float angle);

  /**
   * This ArmourModel can be used as a placeholder whenever a proper one has not
   * been defined. This ArmourModel represents an invincible entity.
   */
  public final static ArmourModel NULL_ARMOUR_MODEL = new ArmourModel(){
    public void reportImpact(Weapon weapon, int impactX, int impactY,
                             float impactDist, float travellingRange,
                             float impactAngle){
      System.out.println("Warning: No ArmourModel registered!");
    }

    public float vulnerability(Weapon weapon, float range, float angle){
      return 0.0F;
    }

  };

}
