package jwars.model.weapons;

import jwars.model.units.InfantrySquad;
import jwars.model.WarModel;
import jwars.model.weapons.ShellType;
import jwars.model.World;

public class InfantryArmourModel implements ArmourModel {

  private final InfantrySquad squad;
  private double toughness = 12;

  public InfantryArmourModel(InfantrySquad squad){
    this.squad = squad;
  }

  public void reportImpact(Weapon weapon, int impactX, int impactY,
                          float impactDist, float travellingRange,
                          float impactAngle){
    double killValue = weapon.CAT.killValue;
    double random = Math.abs(WarModel.WORLD.rand.nextDouble());
    if(weapon.CAT.splashRange < impactDist){
      return;
    }
    double rangeBadnessFactor = 2*weapon.CAT.splashRange/(weapon.CAT.splashRange+impactDist)-1;
    double avgCasualties = killValue * Math.sqrt(squad.getStrength()) * rangeBadnessFactor / toughness;

    if(weapon.CAT.type == ShellType.bullet){
      avgCasualties = avgCasualties * weapon.CAT.effectiveRange / travellingRange * weapon.getSalvoSize();
    } else {
      avgCasualties = 2.4*avgCasualties;
    }
    double casualties = avgCasualties * random * random;
    //World w = WarModel.WORLD;
    //System.out.println("---"+weapon+" firing at "+squad+"---");
    //System.out.println("Range: "+w.toMetres((int)travellingRange)+"\t Eff rng: "+w.toMetres((int)weapon.CAT.effectiveRange));
    //System.out.println("Impactdist "+w.toMetres((int)impactDist));
    //System.out.println("Max cas: "+avgCasualties+"\t Adj cas: "+casualties);
    squad.kill((int)(casualties));
  }

  public float vulnerability(Weapon weapon, float range, float angle){
    return 1F;
  }

}
