package jwars.net;

import java.io.DataInput;
import java.io.DataInputStream;
import java.io.DataOutput;
import java.io.DataOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;
import javax.swing.JOptionPane;

import jwars.Util;
import jwars.model.units.Teams;
import jwars.test.ConnectionListener;
import jwars.test.Player;

public class DefaultClientEventHandler implements ClientEventHandler {

  private final Hashtable<Integer, Player> players = new Hashtable<Integer,Player>();
  private int localClientNumber;
  private List<ConnectionListener> listeners = new ArrayList<ConnectionListener>();

  public DefaultClientEventHandler(){
  }

  public void handShake(int clientNumber, DataInputStream input, DataOutputStream output) throws
      IOException{
    this.localClientNumber = clientNumber;
    //Write local information to server
    output.writeUTF(Util.VERSION);
    output.writeUTF(Util.getUserName());
    output.writeUTF(Teams.GERMANY.NAME_SHORT); //odd place to set default team
    output.flush();
    //boolean accepted = input.readBoolean();
    //if(!accepted){
    //  JOptionPane.showConfirmDialog(null, "Sorry, connection unsuccessful");
    //}
    //Receive information from server about other clients
    int otherClientCount = input.readInt();
    for(int i = 0; i < otherClientCount; i++){
      //Will also register local player because the relevant info is sent by the
      //server. However watch out for equality of Player objects with a possible
      //local representation, i.e. two different player objects representing
      //the same player
      int playerIndex = input.readInt();
      clientJoined(playerIndex, input);
    }
  }

  public void clientRemoved(int clientNumber, DataInput input) throws
      IOException{
    Player player = players.get(clientNumber);
    if(clientNumber == localClientNumber){
      System.out.println("Disconnected!");
      System.exit(0);
    } else {
      Player p = players.remove(clientNumber);
      if(p == null){
        System.out.println("DefaultClientEventHandler: Nonexistent Player "+p+" specified for removal");
      } else {
        System.out.println("DefaultClientEventHandler: Player "+p+" removed");
      }
    }
    for(ConnectionListener listener : listeners){
      listener.playerLeft(player);
    }
  }

  public void clientJoined(int clientNumber, DataInput input) throws
      IOException{
    String name = input.readUTF();
    String teamName = input.readUTF();
    Player player = new Player(name, Teams.getTeamByShortName(teamName));
    assert(players.get(clientNumber)==null);
    players.put(clientNumber, player);
    for(ConnectionListener listener : listeners){
      listener.playerJoined(player);
    }
  }

  public int getPlayerCount(){
    return players.size();
  }

  public Player getPlayer(int id){
    return players.get(id);
  }

  public Player getLocalPlayer(){
    return players.get(this.localClientNumber);
  }

  public boolean isLocalPlayer(Player p){
    return p == getLocalPlayer();
  }

  public void addConnectionListener(ConnectionListener listener){
    listeners.add(listener);
  }

  public void removeConnectionListener(ConnectionListener listener){
    listeners.remove(listener);
  }

  public void clientPropertyChanged(int clientNumber, DataInput input) throws
      IOException{
    String name = input.readUTF();
    String team = input.readUTF();
    Player p = players.get(clientNumber);
    p.setName(name);
    p.setTeam(Teams.getTeamByShortName(team));
    for(ConnectionListener listener : listeners){
      listener.playerPropertyChanged(p);
    }
  }

  public void writeClientProperties(DataOutput output) throws IOException{
    Player local = getLocalPlayer();
    output.writeUTF(local.getName());
    output.writeUTF(local.getTeam().NAME_SHORT);
  }

}
